@@ -9,9 +9,7 @@ use strict;
 use Moose;
 use IPC::SysV qw(S_IRUSR S_IWUSR IPC_CREAT);
 use IPC::SharedMem;
-use Storable qw(freeze thaw);
-use Data::Dumper;
-use Digest::MD5;
+use JSON::MaybeXS;
 use Carp;
 use Config;
 use Fcntl;
@@ -46,18 +44,18 @@ CHI::Driver::SharedMem - Cache data in shared memory
 
 =head1 VERSION
 
-Version 0.14
+Version 0.15
 
 =cut
 
-our $VERSION = '0.14';
+our $VERSION = '0.15';
 
 # FIXME - get the pod documentation right so that the layout of the memory
 # area looks correct in the man page
 
 =head1 SYNOPSIS
 
-L<CHI> driver which stores data in shared memory objects for persistently
+L<CHI> driver which stores data in shared memory objects for persistence
 over processes.
 Size is an optional parameter containing the size of the shared memory area,
 in bytes.
@@ -96,7 +94,8 @@ The shared memory area is stored thus:
 
 =head2 store
 
-Stores an object in the cache
+Stores an object in the cache.
+The data are serialized into JSON.
 
 =cut
 
@@ -249,7 +248,7 @@ sub _data {
 	my($self, $h) = @_;
 
 	if(defined($h)) {
-		my $f = freeze($h);
+		my $f = JSON::MaybeXS->new()->ascii()->encode($h);
 		my $cur_size = length($f);
 		$self->shm()->write($f, $Config{intsize}, $cur_size);
 		$self->_data_size($cur_size);
@@ -259,7 +258,7 @@ sub _data {
 	unless($cur_size) {
 		return {};
 	}
-	return thaw($self->shm()->read($Config{intsize}, $cur_size));
+	return JSON::MaybeXS->new()->ascii()->decode($self->shm()->read($Config{intsize}, $cur_size));
 }
 
 =head2 BUILD
@@ -284,29 +283,49 @@ it's safe to remove it and reclaim the memory.
 =cut
 
 sub DEMOLISH {
+	if(defined($^V) && ($^V ge 'v5.14.0')) {
+		return if ${^GLOBAL_PHASE} eq 'DESTRUCT';	# >= 5.14.0 only
+	}
 	my $self = shift;
 
 	if($self->shmkey()) {
 		my $cur_size;
+		$self->_lock(type => 'write');
 		if(scalar($self->get_namespaces())) {
-			$self->_lock(type => 'read');
 			$cur_size = $self->_data_size();
-			$self->_unlock();
 		} else {
-			$self->_lock(type => 'write');
 			$self->_data_size(0);
-			$self->_unlock();
 			$cur_size = 0;
 		}
-		$self->shm()->detach();
-		# We could scan the cache and see if all has expired.
-		# If it has, then the cache could be removed if nattch = 0.
-		unless($cur_size) {
-			my $stat = $self->shm()->stat();
-			if(defined($stat) && ($stat->nattch() == 0)) {
+		my $stat = $self->shm()->stat();
+		if($cur_size == 0) {
+			if(defined($stat) && ($stat->nattch() == 1)) {
+				$self->shm()->detach();
 				$self->shm()->remove();
 			}
+		# } elsif(defined($stat) && ($stat->nattch() == 1)) {
+			# # Can the cache and see if all has expired.
+			# # If it has, then the cache could be removed if nattch = 1 (us)
+			# my $can_remove = 1;
+			# my @namespaces = $self->get_namespaces();
+			# foreach my $namespace(@namespaces) {
+				# my @keys = $self->get_keys($namespace);
+				# foreach my $key(@keys) {
+					# if($self->is_valid($key)) {	# <------- gives substr error in CHI
+						# $can_remove = 0;
+					# }
+				# }
+			# }
+			# if($can_remove) {
+				# $self->shm()->detach();
+				# $self->shm()->remove();
+			# } else {
+				# $self->shm()->detach();
+			# }
+		} else {
+			$self->shm()->detach();
 		}
+		$self->_unlock();
 	}
 }
 
@@ -320,51 +339,46 @@ Please report any bugs or feature requests to C<bug-chi-driver-sharedmem at rt.c
 the web interface at L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=CHI-Driver-SharedMem>.  I will be notified, and then you'll
 automatically be notified of progress on your bug as I make changes.
 
-
 =head1 SEE ALSO
 
-CHI, IPC::SharedMem
+L<CHI>, L<IPC::SharedMem>
 
+=cut
 
 =head1 SUPPORT
 
 You can find documentation for this module with the perldoc command.
 
-    perldoc CHI::Driver::SharedMem
-
+    perldoc CHI::Driver::SharedMemory
 
 You can also look for information at:
 
 =over 4
 
+=item * MetaCPAN
+
+L<https://metacpan.org/dist/CHI-Driver-SharedMem>
+
 =item * RT: CPAN's request tracker
 
-L<http://rt.cpan.org/NoAuth/Bugs.html?Dist=CHI-Driver-SharedMem>
+L<https://rt.cpan.org/NoAuth/Bugs.html?Dist=CHI-Driver-SharedMemory>
 
-=item * AnnoCPAN: Annotated CPAN documentation
+=item * CPAN Testers' Matrix
 
-L<http://annocpan.org/dist/CHI-Driver-SharedMem-Info>
+L<http://matrix.cpantesters.org/?dist=CHI-Driver-SharedMemory>
 
-=item * CPAN Ratings
+=item * CPAN Testers Dependencies
 
-L<http://cpanratings.perl.org/d/CHI-Driver-SharedMem>
-
-=item * Search CPAN
-
-L<http://search.cpan.org/dist/CHI-Driver-SharedMem>
+L<http://deps.cpantesters.org/?module=CHI::Driver::SharedMemory>
 
 =back
 
-
-=head1 ACKNOWLEDGEMENTS
-
-
 =head1 LICENSE AND COPYRIGHT
 
-Copyright 2012-2014 Nigel Horne.
+Copyright 2010-2023 Nigel Horne.
 
-This program is released under the following licence: GPL
+This program is released under the following licence: GPL2
 
 =cut
 
-1; # End of CHI::Driver::SharedMem
+1;
