@@ -37,7 +37,7 @@ Declare the driver being tested
 =cut
 
 sub testing_driver_class {
-	'CHI::Driver::SharedMem'
+	return 'CHI::Driver::SharedMem';
 }
 
 =head2 new_cache_options
@@ -49,8 +49,8 @@ sub new_cache_options {
 	return (
 	    $self->SUPER::new_cache_options(),
 	    driver => '+CHI::Driver::SharedMem',
-	    size => 16 * 1024,
-	    shmkey => 12344321,
+	    size => $main::size,
+	    shmkey => $main::shmkey,
 	);
 }
 
